/* $Id: perfctr.h,v 1.36 2002/10/20 21:51:06 mikpe Exp $
 * Performance-Monitoring Counters driver
 *
 * Copyright (C) 1999-2002  Mikael Pettersson
 */
#ifndef _LINUX_PERFCTR_H
#define _LINUX_PERFCTR_H

#include <asm/perfctr.h>

struct perfctr_info {
	char version[32];
	unsigned char nrcpus;
	unsigned char cpu_type;
	unsigned char cpu_features;
	unsigned long cpu_khz;
};

/* cpu_type values */
#define PERFCTR_X86_GENERIC	0	/* any x86 with rdtsc */
#define PERFCTR_X86_INTEL_P5	1	/* no rdpmc */
#define PERFCTR_X86_INTEL_P5MMX	2
#define PERFCTR_X86_INTEL_P6	3
#define PERFCTR_X86_INTEL_PII	4
#define PERFCTR_X86_INTEL_PIII	5
#define PERFCTR_X86_CYRIX_MII	6
#define PERFCTR_X86_WINCHIP_C6	7	/* no rdtsc */
#define PERFCTR_X86_WINCHIP_2	8	/* no rdtsc */
#define PERFCTR_X86_AMD_K7	9
#define PERFCTR_X86_VIA_C3	10	/* no pmc0 */
#define PERFCTR_X86_INTEL_P4	11	/* model 0 and 1 */
#define PERFCTR_X86_INTEL_P4M2	12	/* model 2 and above */

/* cpu_features flag bits */
#define PERFCTR_FEATURE_RDPMC	0x01
#define PERFCTR_FEATURE_RDTSC	0x02
#define PERFCTR_FEATURE_PCINT	0x04

/* user's view of mmap:ed virtual perfctr */
struct vperfctr_state {
	unsigned int magic;
	int si_signo;
	struct perfctr_cpu_state cpu_state;
};

/* `struct vperfctr_state' binary layout version number */
#define VPERFCTR_STATE_MAGIC	0x0201	/* 2.1 */
#define VPERFCTR_MAGIC	((VPERFCTR_STATE_MAGIC<<16)|PERFCTR_CPU_STATE_MAGIC)

/* parameter in VPERFCTR_CONTROL command */
struct vperfctr_control {
	int si_signo;
	struct perfctr_cpu_control cpu_control;
	unsigned long preserve;
};

/*
 * sys_perfctr(unsigned int cmd, int whom, void *arg)
 *
 *	cmd			   whom, arg
 */
#define PERFCTR_INFO	 0	/* 0, struct perfctr_info* */
#define VPERFCTR_SAMPLE	 1	/* fd, NULL */
#define VPERFCTR_UNLINK	 2	/* fd, NULL */
#define VPERFCTR_CONTROL 3	/* fd, struct vperfctr_control* */
#define VPERFCTR_IRESUME 4	/* fd, NULL */
#define VPERFCTR_OPEN	 5	/* pid, NULL */
#define VPERFCTR_CREAT	 6	/* pid, NULL */

#ifdef __KERNEL__

extern struct perfctr_info perfctr_info;

/*
 * Virtual per-process performance-monitoring counters.
 */
struct vperfctr;	/* opaque */

#ifdef CONFIG_PERFCTR_VIRTUAL

/* process management operations */
extern struct vperfctr *__vperfctr_copy(struct vperfctr*);
extern void __vperfctr_exit(struct vperfctr*);
extern void __vperfctr_suspend(struct vperfctr*);
extern void __vperfctr_resume(struct vperfctr*);
extern void __vperfctr_sample(struct vperfctr*);

static inline void perfctr_copy_thread(struct thread_struct *thread)
{
	thread->perfctr = NULL;
}

static inline void perfctr_exit_thread(struct thread_struct *thread)
{
	struct vperfctr *perfctr;
	perfctr = thread->perfctr;
	if( perfctr ) {
		thread->perfctr = NULL;
		__vperfctr_exit(perfctr);
	}
}

static inline void perfctr_suspend_thread(struct thread_struct *prev)
{
	struct vperfctr *perfctr;
	perfctr = prev->perfctr;
	if( perfctr )
		__vperfctr_suspend(perfctr);
}

/* PRE: next is current */
static inline void perfctr_resume_thread(struct thread_struct *next)
{
	struct vperfctr *perfctr;
	perfctr = next->perfctr;
	if( perfctr )
		__vperfctr_resume(perfctr);
}

static inline void perfctr_sample_thread(struct thread_struct *thread)
{
#ifdef CONFIG_SMP
	struct vperfctr *perfctr;
	perfctr = thread->perfctr;
	if( perfctr )
		__vperfctr_sample(perfctr);
#endif
}

#else	/* !CONFIG_PERFCTR_VIRTUAL */

static inline void perfctr_copy_thread(struct thread_struct *t) { }
static inline void perfctr_exit_thread(struct thread_struct *t) { }
static inline void perfctr_suspend_thread(struct thread_struct *t) { }
static inline void perfctr_resume_thread(struct thread_struct *t) { }
static inline void perfctr_sample_thread(struct thread_struct *t) { }

#endif	/* CONFIG_PERFCTR_VIRTUAL */

#endif	/* __KERNEL__ */

#endif	/* _LINUX_PERFCTR_H */
